#include "fpsplayer.h"

FPSPlayer* FPSPlayer::thePlayer = NULL;
unsigned int PlayerCharacter::frameListenerPriority = 1;
float TrollCharacter::MaxHealth = 350;
CharacterManager* CharacterManager::mSingleton = 0;

void CharacterManager::levelUP()
{
		//player level up
		FPSPlayer::thePlayer->levelUp();
		//add 2 new trolls
		static int trollID = 2;
		static int kills = 0;
		kills++;
		
		OverlayElement* killsTextBox = OverlayManager::getSingleton().getOverlayElement("Level");
			killsTextBox->setCaption("Kills: "+StringConverter::toString(kills));

		for(int i = 0; i < 2; i++)
		{
			TrollCharacter* troll = new TrollCharacter("troll" + StringConverter::toString(trollID),
															mScene,
															mSceneManager,
															Vector3(1.5, 3.0, 1.2),
															Vector3(25, 10, 25 - i * 10), false);

			troll->addPlayerMesh("troll.mesh", 0.05, false);
			troll->addWeaponModel("trollMace.mesh", "RightHand");
			OgreIlluminationManager::getSingleton().initTechniques(troll->getMeshEntity());
			trollID++;			
		}


		
}

class MyTrigger : public Trigger
{
public:
	MyTrigger(const NxString& meshIdentifier, Scene* scene, ShapeDescription* shape, const NxOgre::Pose& pose, ActorParams params = ""):
	  Trigger(meshIdentifier, scene, shape, pose, params)
	  {
	  }

	 void onEnter(Actor* a)
	 {
		 PlayerCharacter* trigger = CharacterManager::getSingleton().getTrigger(this);
		 PlayerCharacter* target = CharacterManager::getSingleton().getTarget(a);
		
		 if(target != trigger && target == FPSPlayer::thePlayer)
			target->addHit(trigger->getHit());

	 }
};

class FireBall : public Trigger
{
public:
	FireBall(String& name,
		     SceneManager* sm,
			 SceneNode* meshNode,
			 Bone* attachBone,
			 const NxString& meshIdentifier,
			 Scene* scene,
			 ShapeDescription* shape,
			 const NxOgre::Pose& pose,
			 ActorParams params = "")
	: Trigger(meshIdentifier, scene, shape, pose, params)
	  {
		  mFire=false;
		  isActive = false;
		 mFireBall = sm->createParticleSystem("FPS_PLAYER_FIREBALL", "Moria/FireBall");
			
		 mFireBallLight = sm->createLight(mName+"FireBallLight");
		 mFireBallLight->setType(Light::LT_POINT);
		 mFireBallLight->setCastShadows(true);
		 mFireBallLight->setDiffuseColour(ColourValue(0.95,0.98,0.8));
		 mFireBallLight->setPowerScale(200);
		 mFireBallLight->setAttenuation(200.0, 0, 0, 6);	
		 mFireBallLight->setVisible(false);

		 attachNode = meshNode->createChildSceneNode(name + "_AttachNode");
		 fireBallNode = attachNode->createChildSceneNode(name + "_FireBallNode");
		// fireBallNode->attachObject(mFireBallLight);
		 BillboardSet* bbs = ((BillboardParticleRenderer*)mFireBall->getRenderer())->getBillboardSet();
		 fireBallNode->attachObject(mFireBall);
		 bbs->setMaterialName(mFireBall->getMaterialName());
			OgreIlluminationManager::getSingleton().initTechniques(
				bbs,
				mFireBall);
		 fireBallNode->detachObject(mFireBall);
		 mFireBall->setVisible(false);
		 sceneManager = sm;
		 this->attachBone = attachBone;

		/* explosionNode = sm->getRootSceneNode()->createChildSceneNode(name + "_EXPLOSION_NODE");
		 mExplosion = sm->createParticleSystem("FPS_PLAYER_EXPLOSION", "Moria/Explosion");
		 explosionNode->attachObject(mExplosion);
		 bbs = ((BillboardParticleRenderer*)mExplosion->getRenderer())->getBillboardSet();
		 bbs->setMaterialName(mExplosion->getMaterialName());
			OgreIlluminationManager::getSingleton().initTechniques(
				bbs,
				mExplosion);*/
         		
	 }
   void update(float dt)
   {
	   if(isActive)
	   {
		/*if(explodeTimer.getMilliseconds() > 1000)
			mExplosion->getEmitter(0)->setEmissionRate(0);*/
		   
		Vector3 pos = attachBone->getWorldPosition();
		Quaternion orientation = attachBone->getWorldOrientation();

		attachNode->setPosition(pos);
		attachNode->setOrientation(orientation);

		if(mFire)
		{
			Vector3 testCenter = position - direction;
			bool collidedWithStatic = getScene()->getNxScene()->checkOverlapSphere( 
					NxSphere(NxVec3(testCenter.x,testCenter.y,testCenter.z), 0.4),  
					NX_STATIC_SHAPES);
			position += direction * dt * 10.0;
			fireBallNode->setPosition(position);
			
			if(collidedWithStatic)
				explode();		
 		}
		  
		this->setGlobalPosition(fireBallNode->getWorldPosition());
		this->setGlobalOrientation(fireBallNode->getWorldOrientation());
	   }
   }
   void fire()
   {
	   mFire = true;
	   position = fireBallNode->getWorldPosition();
	   direction = FPSPlayer::getPlayer()->getDirection();
	   //position += direction * 2.0;	

	   attachNode->removeChild(fireBallNode);
	   sceneManager->getRootSceneNode()->addChild(fireBallNode);
  }
   void activate()
   {
	   isActive = true;
	   mFireBall->setVisible(true);
	  // mFireBallLight->setVisible(true);
	   fireBallNode->attachObject(mFireBall);
	   fireBallNode->attachObject(mFireBallLight);
	   //mFireBallLight->setPowerScale(200);
	   CompositorManager::getSingleton().setCompositorEnabled(
		   OgreIlluminationManager::getSingleton().getMainViewport(),
			"FireHeatCompositor", true);
	   
   }
   void deactivate()
   {
	   isActive = false;
	   fireBallNode->detachObject(mFireBall);
	   fireBallNode->detachObject(mFireBallLight);	   
	   //mFireBallLight->setPowerScale(0);
	    mFireBall->setVisible(false);
	//   mFireBallLight->setVisible(false);
	   CompositorManager::getSingleton().setCompositorEnabled(
		   OgreIlluminationManager::getSingleton().getMainViewport(),
			"FireHeatCompositor", false);
   }

   void onEnter(Actor* a)
   {
	 PlayerCharacter* target = CharacterManager::getSingleton().getTarget(a);
	 if(target != FPSPlayer::thePlayer)
	 {
		 if(mFire)
		 {
			 if(target)
		 		target->addHit(30);
		     explode();
		 }
	 }
 }

protected:
   Vector3 position;
   Vector3 direction;
	bool mFire;
	bool isActive;
	SceneNode* fireBallNode;
	SceneNode* explosionNode;
	SceneNode* attachNode;
	ParticleSystem* mFireBall;
	ParticleSystem* mExplosion;
	Light* mFireBallLight;
	Bone* attachBone;
	SceneManager* sceneManager;	
	Timer explodeTimer;

	void explode()
	{
		/*mExplosion->getEmitter(0)->setEmissionRate(100);
		explodeTimer.reset();
		mExplosion->getEmitter(0)->setPosition(position);*/
		sceneManager->getRootSceneNode()->removeChild(fireBallNode);
		attachNode->addChild(fireBallNode);
		fireBallNode->setPosition(2.8, -2.28, 0.47);
		mFire = false;		
	}
};

class MyCharMoveController : public CharacterController
{
public:
	MyCharMoveController(float speed = 1.5){movespeed = speed;}
	float movespeed;
	void move(NxVec3 &out, NxVec3 &moveVector, NxQuat &direction, NxVec3 &g, float t, Character*)
	{
		NxVec3 dir = direction.rot(moveVector);
		out = ((dir * movespeed) + g) * t;		
	}
	NxControllerAction onShapeHit(const NxControllerShapeHit & hit)
	{
		return CharacterController::onShapeHit(hit);
	}
	NxControllerAction onControllerHit(const NxControllersHit & hit)
	{
		return CharacterController::onControllerHit(hit);
	}
};

PlayerCharacter::PlayerCharacter(String name, Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, bool debug)
{
	mCurrentAnimState = PLAYERANIMSTATE_STAND;
	isAlive = true;
	mName = name;
	mPlayerDimensions = dimensions;
	mDebug = debug;
	mSceneManager = sceneManager;
	mScene = scene;
	CharacterParams cp;
	cp.setToDefault();
	cp.mDimensions = NxVec3(dimensions.x, dimensions.y, dimensions.z);
	cp.mType = CharacterParams::CT_Box;
	mPlayerCharacter = mScene->createCharacter(mName, startPosition, cp); 
	NxActor* a = mPlayerCharacter->getNxController()->getActor();
	CharacterManager::getSingleton().addTarget((Actor*) a->userData, this);
	mPlayerCharacter->createNode();
	mPlayerNode = mPlayerCharacter->getNode();
	
	mCharacterController = 0;
	mCharacterController = new MyCharMoveController(2.0);
	mPlayerCharacter->setMovementVectorController(mCharacterController);

	if(mDebug)
	{
		
		mPlayerBoundDebugNode = mPlayerNode->createChildSceneNode(mName + "_BoundDebugNode");
		mPlayerBoundDebugNode->setScale(Vector3(dimensions.x, dimensions.y, dimensions.z));
	//	mPlayerBoundDebugNode->attachObject(mSceneManager->createEntity(mName + "_BoundDebug", "cube.mesh"));
		
		//mPlayerNode->setScale(Vector3(dimensions.x, dimensions.y, dimensions.z));
		//mPlayerNode->attachObject(mSceneManager->createEntity(mName + "_BoundDebug", "cube.mesh"));
	}

	mCharacterMesh_AnimState = 0;
	mForceAnimChange = false;
	mWeapon = 0;
	mWeaponBound = 0;
	mWeaponBoundDebugNode = 0;
	mCharacterMeshEntity = 0;
	mWeaponEntity = 0;
	mWeaponAttachBone = 0;
    mWeaponNode = 0;
	mWeaponBoundNode = 0;
	mWeaponBoundDebugNode = 0;
	mPlayerBoundDebugNode = 0;
	mWeaponBound = 0;
	mWeaponNode2 = 0;
		
	this->setPriority(frameListenerPriority++);
	Root::getSingleton().addFrameListener(this);

	//mPlayerCharacter->setCharacterCollisionGroup(NX_CHARACTER_COL_GROUP_CHARACTER);
	//mScene->getNxScene()->setGroupCollisionFlag(NX_CHARACTER_COL_GROUP_CHARACTER, NX_CHARACTER_COL_GROUP_NON_COLLIDABLE,false);
}

TrollCharacter::TrollCharacter(String name, Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, bool debug)
: PlayerCharacter(name, scene, sceneManager, dimensions, startPosition, debug)
{
	mHealth = MaxHealth;
}

FPSPlayer::FPSPlayer(Scene* scene, SceneManager* sceneManager, Vector3 dimensions, Vector3 startPosition, float cameraHeight, Camera* cam, bool debug)
: PlayerCharacter("FPSPlayerCharacter", scene, sceneManager, dimensions, startPosition, debug)
{
	mMaxHealth = 50;
	mMaxStrength = 100;
	mMaxManna = 100;
	mStrength = mMaxStrength;
	mHealth = mMaxHealth;
	mManna = mMaxManna;
	
	mCameraHeight = cameraHeight - dimensions.y / 2.0;
	mCamera = cam;
	mPlayerNode->attachObject(cam); 
	mCamera->rotate(Vector3(0,1,0), Radian(Degree(180)));
	
	thePlayer = this;
	mFireBall = 0;
	
}

FPSPlayer::~FPSPlayer(void)
{
}

void PlayerCharacter::addPlayerMesh(String meshname, float meshscale, bool rotateModel)
{
	mCharacterMeshEntity = mSceneManager->createEntity(mName, meshname);
	mCharacterMeshEntity->setCastShadows(true);
	mMeshScale = meshscale;
	mCurrentAnimState = PLAYERANIMSTATE_STAND;
	mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_STAND));
	mCharacterMesh_AnimState->setEnabled(true);
	mCharacterMesh_AnimState->setLoop(true);
	SceneNode* meshNode = mPlayerNode->createChildSceneNode(mName + "_MeshNode");
	meshNode->attachObject(mCharacterMeshEntity);
	meshNode->setPosition(0 ,-mPlayerDimensions.y / 2.0, 0);
	meshNode->setScale(meshscale, meshscale, meshscale);
	if(rotateModel)
	meshNode->rotate(Vector3::UNIT_Y, Degree(180.0));	
}

void FPSPlayer::addPlayerMesh(String meshname, float meshscale, bool rotateModel)
{
	mCharacterMeshEntity = mSceneManager->createEntity(mName, meshname);
	mCharacterMeshEntity->setCastShadows(false);
	mMeshScale = meshscale;
	mCurrentAnimState = PLAYERANIMSTATE_STAND;
	mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_STAND));
	mCharacterMesh_AnimState->setEnabled(true);
	mCharacterMesh_AnimState->setLoop(true);
	SceneNode* FPSArmNode = mPlayerNode->createChildSceneNode(mName+"_ArmNode");
	FPSArmNode->attachObject(mCharacterMeshEntity); 
	FPSArmNode->setPosition(0,mCameraHeight,0);
	FPSArmNode->setScale(meshscale, meshscale, meshscale);
	if(rotateModel)
	FPSArmNode->rotate(Vector3::UNIT_Y, Degree(180.0));	

	mFireBall = new FireBall(mName+"_FIREBALL",
							 mSceneManager,
							 FPSArmNode,
							 mCharacterMeshEntity->getSkeleton()->getBone("joint4"),
							 mName+"FireBallTrigger",
							 mScene,
							 new SphereShape(0.5, "trigger: yes"),
							 Vector3(0,0,0),
							 "mass: 0, density: 0");
	mScene->addTrigger(mFireBall);
}

void PlayerCharacter::addWeaponModel(String meshname, String attachBoneName)
{
	
	mWeaponEntity = mSceneManager->createEntity(mName + "_Weapon", meshname);
	mWeaponAttachBone = mCharacterMeshEntity->getSkeleton()->getBone(attachBoneName);
	mWeaponNode = mCharacterMeshEntity->getParentSceneNode()->createChildSceneNode(mName + "_WeaponNode");	
	mWeaponNode->attachObject(mWeaponEntity);
	mWeaponEntity->setCastShadows(true);

	AxisAlignedBox bb = mWeaponEntity->getBoundingBox();

	Vector3 max = bb.getMaximum();
	Vector3 min = bb.getMinimum();
	Vector3 center = (min + max) / 2.0;
	
	mWeaponBoundNode = mWeaponNode->createChildSceneNode(mName + "_WeaponBoundNode");
	mWeaponBoundNode->setPosition(center);
/*
	SceneNode* ArmNode2 = mPlayerNode->createChildSceneNode();
	ArmNode2->setPosition(0,mCameraHeight , -mPlayerDimensions.z);
	ArmNode2->rotate(Vector3::UNIT_Y, Degree(180.0));
	mWeaponNode2 = ArmNode2->createChildSceneNode();
	mWeaponBoundNode = mWeaponNode2->createChildSceneNode();
	mWeaponBoundNode->setPosition(center * mMeshScale );
*/	

	max *= mMeshScale;
	min *= mMeshScale;
	float dimX = max.x-min.x;
	float dimY = max.y-min.y;
	float dimZ = max.z-min.z;
	Vector3 dim(dimX, dimY, dimZ);

	mWeaponBound = new MyTrigger(mName + "_WeaponTrigger", mScene, new CubeShape(dimX, dimY, dimZ, "trigger: yes"), Vector3(0,0,0), "mass: 0, density: 0");
	mScene->addTrigger((Trigger*)mWeaponBound);

	CharacterManager::getSingleton().addTrigger(mWeaponBound, this);	
	if(mDebug)
	{
		mWeaponBoundDebugNode = mSceneManager->getRootSceneNode()->createChildSceneNode(mName+"_WeaponBoundDebugNode");
		mWeaponBoundDebugNode->setScale(dim);
		mWeaponBoundDebugNode->attachObject(mSceneManager->createEntity(mName+"_WeaponBoundDebug", "cube.mesh"));
	}

	//mWeaponBound->setCharacterCollisionGroup(NX_CHARACTER_COL_GROUP_NON_COLLIDABLE);	
}

void FPSPlayer::addWeaponModel(String meshname, String attachBoneName)
{	
	mWeaponEntity = mSceneManager->createEntity(mName + "_Weapon", meshname);
	mWeaponAttachBone = mCharacterMeshEntity->getSkeleton()->getBone(attachBoneName);
	mWeaponNode = mCharacterMeshEntity->getParentSceneNode()->createChildSceneNode(mName + "_WeaponNode");	
	mWeaponNode->attachObject(mWeaponEntity);
	mWeaponEntity->setCastShadows(false);

	AxisAlignedBox bb = mWeaponEntity->getBoundingBox();

	mWeaponBoundNode = mPlayerNode->createChildSceneNode(mName + "_WeaponBoundNode");
	mWeaponBoundNode->setPosition(0,1.5,0.5);
	
	mWeaponBound = new MyTrigger(mName + "_WeaponTrigger", mScene, new CubeShape(0.5, 0.5, 0.5, "trigger: yes"), Vector3(0,0,0), "mass: 0, density: 0");
	mScene->addTrigger((Trigger*)mWeaponBound);

	CharacterManager::getSingleton().addTrigger(mWeaponBound, this);	
	if(mDebug)
	{
		mWeaponBoundDebugNode = mSceneManager->getRootSceneNode()->createChildSceneNode(mName+"_WeaponBoundDebugNode");
		mWeaponBoundDebugNode->setScale(0.5,0.5,0.5);
	//	mWeaponBoundDebugNode->attachObject(mSceneManager->createEntity(mName+"_WeaponBoundDebug", "cube.mesh"));
	}

	//mWeaponBound->setCharacterCollisionGroup(NX_CHARACTER_COL_GROUP_NON_COLLIDABLE);	
}

void TrollCharacter::doAction(float dt)
{
	if(mCurrentAnimState == PLAYERANIMSTATE_DEATH)
		return;

	FPSPlayer* thePlayer = FPSPlayer::getPlayer();

	if(thePlayer)
	{
		Vector3 playerPos = thePlayer->getPosition();
		Vector3 myPos = mPlayerNode->getWorldPosition();
		static Vector3 dir = Vector3(0,0,1);
		Vector3 lastDir = dir;
		dir = playerPos - myPos;
		dir.y = 0;

		float dist = dir.length();

		dir.normalise();
		Vector3 right = -dir.crossProduct(Vector3::UNIT_Y);
		Quaternion q; 			
		q.FromAxes(right,Vector3::UNIT_Y, dir);
		mPlayerCharacter->setDirection(q);
		
		if(dist < 1.8)
		{
			if(dist < 1.2)
			{
				setAnimState(PLAYERANIMSTATE_WALK);
				if(mCurrentAnimState != PLAYERANIMSTATE_DEATH &&
					mCurrentAnimState != PLAYERANIMSTATE_ATTACK &&
					mCurrentAnimState != PLAYERANIMSTATE_PAIN)
				{
					if(mCharacterController )
						((MyCharMoveController*)mCharacterController)->movespeed = 1.5;
					mPlayerCharacter->addMovement(Character::DR_Backward);
					
				}		
				FPSPlayer::thePlayer->addCharacterInRange(this);
			}
			else
			{
				if(Math::UnitRandom()>0.6)
				{
					if(Math::ACos(dir.dotProduct(lastDir)).valueDegrees() > 2.0)
						setAnimState(PLAYERANIMSTATE_TURN);
					else
						setAnimState(PLAYERANIMSTATE_STAND);
				}
				else
				{
					setAnimState(PLAYERANIMSTATE_ATTACK);
				}
			}
		}
		else
		{
			FPSPlayer::thePlayer->removeCharacterInRange(this);

			setAnimState(PLAYERANIMSTATE_WALK);
			if(mCurrentAnimState != PLAYERANIMSTATE_DEATH &&
				mCurrentAnimState != PLAYERANIMSTATE_ATTACK &&
				mCurrentAnimState != PLAYERANIMSTATE_PAIN)
			{
				if(mCharacterController )
					((MyCharMoveController*)mCharacterController)->movespeed = 1.5;
				mPlayerCharacter->addMovement(Character::DR_Forward);
				
			}		
		}	
	}	
}

void TrollCharacter::addHit(float hit)
{
	if(hit == 0 || !isAlive)
		return;

	PlayerCharacter::addHit(hit);

	if(mHealth <= 0)//death
	{
		isAlive = false;
		setAnimState(PLAYERANIMSTATE_DEATH);

		//TODO:  free character controller
		mScene->destroyCharacter(mName);
		CharacterManager::getSingleton().levelUP();
	}
	else//pain
	{
		setAnimState(PLAYERANIMSTATE_PAIN);
	}
}

void  TrollCharacter::setAnimState(PlayerAnimState as)
{
	if(!mCharacterMeshEntity)
		return;

	if(mCurrentAnimState == PLAYERANIMSTATE_DEATH)
		return;

	switch(as)	
	{
		case PLAYERANIMSTATE_STAND:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_STAND || mForceAnimChange)
			{
				//wait until anim finishes
				if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop())
				{
					mCurrentAnimState = PLAYERANIMSTATE_STAND;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_STAND));
					mCharacterMesh_AnimState->setLoop(true);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_IDLE:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_IDLE || mForceAnimChange)
			{
				//wait until anim finishes
				if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop() || mCurrentAnimState == PLAYERANIMSTATE_IDLE)
				{
					mCurrentAnimState = PLAYERANIMSTATE_IDLE;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_IDLE));
					mCharacterMesh_AnimState->setTimePosition(0);
					mCharacterMesh_AnimState->setLoop(false);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_WALK:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_WALK)
			{
				if(mCurrentAnimState == PLAYERANIMSTATE_IDLE || mCurrentAnimState == PLAYERANIMSTATE_STAND || mCurrentAnimState == PLAYERANIMSTATE_TURN 
					|| mCharacterMesh_AnimState->getTimePosition() >= mCharacterMesh_AnimState->getLength())
				{
					mCurrentAnimState = PLAYERANIMSTATE_WALK;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_WALK));
					mCharacterMesh_AnimState->setLoop(true);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_TURN:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_TURN)
			{
				if(mCurrentAnimState == PLAYERANIMSTATE_IDLE || mCurrentAnimState == PLAYERANIMSTATE_STAND)
				{
					int turnType = 0;
					if(Math::UnitRandom() > 0.5)
					 turnType = 1;

					mCurrentAnimState = PLAYERANIMSTATE_TURN;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_TURN, turnType));
					mCharacterMesh_AnimState->setLoop(true);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_ATTACK:
		{
			if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop() || mCurrentAnimState == PLAYERANIMSTATE_IDLE)
			{
			    int attackType = 0;
				if(Math::UnitRandom() > 0.5)
					 attackType = 1;

				mCurrentAnimState = PLAYERANIMSTATE_ATTACK;
				mCharacterMesh_AnimState->setEnabled(false);
				mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_ATTACK, attackType));
				mCharacterMesh_AnimState->setTimePosition(0);
				mCharacterMesh_AnimState->setLoop(false);
				mCharacterMesh_AnimState->setEnabled(true);
			}
			break;
		}
		case PLAYERANIMSTATE_PAIN:
		{
			if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop()
					//mCurrentAnimState != PLAYERANIMSTATE_PAIN
					)
			{
				int painType = 0;
				if(Math::UnitRandom() > 0.5)
					 painType = 1;

			    mCurrentAnimState = PLAYERANIMSTATE_PAIN;
				mCharacterMesh_AnimState->setEnabled(false);
				mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_PAIN, painType));
				mCharacterMesh_AnimState->setTimePosition(0);
				mCharacterMesh_AnimState->setLoop(false);
				mCharacterMesh_AnimState->setEnabled(true);
			}
			break;
		}
		case PLAYERANIMSTATE_DEATH:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_DEATH)
			{
				int deathType = 0;
				if(Math::UnitRandom() > 0.5)
					 deathType = 1;

			    mCurrentAnimState = PLAYERANIMSTATE_DEATH;
				mCharacterMesh_AnimState->setEnabled(false);
				mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_DEATH, deathType));
				mCharacterMesh_AnimState->setTimePosition(0);
				mCharacterMesh_AnimState->setLoop(false);
				mCharacterMesh_AnimState->setEnabled(true);
			}
			break;
		}		
	}

	if(mForceAnimChange)
		mForceAnimChange = false;
}

String TrollCharacter::getAnimStateName(PlayerAnimState as,  int option)
{
	switch (as)
	{
		case PLAYERANIMSTATE_IDLE:
		{
			if(option)
				return "turn_idle1";
			else
				return "turn_idle2";
		}
		case PLAYERANIMSTATE_TURN:
		{
			if(option)
				return "turn_idle1";
			else
				return "turn_idle2";
		}
		case PLAYERANIMSTATE_STAND:
		{
			return "stand";			
		}
		case PLAYERANIMSTATE_WALK:
		{
				return "walk";			
		}
		case PLAYERANIMSTATE_ATTACK:
		{
			if(option)
			 return "attack1";				
			else
			 return "attack2";				
		}
		case PLAYERANIMSTATE_PAIN:
		{
			if(option)
			 return "pain1";				
			else
			 return "pain2";				
		}
		case PLAYERANIMSTATE_DEATH:
		{
			if(option)
			 return "death1";				
			else
			 return "death2";				
		}
	}
	return "";
}

bool PlayerCharacter::frameStarted(const FrameEvent& evt)
{
//	LogManager::getSingleton().logMessage("CHARACTERPLAYER_FRAMESTARTED");
	doAction(evt.timeSinceLastFrame);

	float dt = evt.timeSinceLastFrame;
		
	if(mCharacterMesh_AnimState)
	 mCharacterMesh_AnimState->addTime(dt * getAnimSpeed());	
	
	if(mCharacterMeshEntity)
	this->mCharacterMeshEntity->_updateAnimation();

	if(mWeaponEntity)
	{
		mWeaponNode->setPosition(mWeaponAttachBone->getWorldPosition());
		mWeaponNode->setOrientation(mWeaponAttachBone->getWorldOrientation());

		Vector3 translation = mWeaponBoundNode->getWorldPosition();
		Quaternion orientation = mWeaponBoundNode->getWorldOrientation();
		
		mWeaponBound->setGlobalOrientation(orientation);		
		mWeaponBound->setGlobalPosition(translation);

		if(mDebug)
		{
			mWeaponBoundDebugNode->setPosition(translation);
			mWeaponBoundDebugNode->setOrientation(orientation);
		}
	}

	return true;
}

void FPSPlayer::processKeyInputs(InputReader* inputDevice)
{	
	if(mHealth > 0)
	{
		mMoving = false;
		if (inputDevice->isKeyDown(KC_W))
		{ 
			mPlayerCharacter->addMovement(Character::DR_Forward); 
			mMoving = true;
		} 	
		if (inputDevice->isKeyDown(KC_S))
		{ 
			mPlayerCharacter->addMovement(Character::DR_Backward);
			mMoving = true;
		} 
		if (inputDevice->isKeyDown(KC_A))
		{ 
			mPlayerCharacter->addMovement(Character::DR_StepRight);
			mMoving = true;
		} 
		if (inputDevice->isKeyDown(KC_D))
		{ 
			mPlayerCharacter->addMovement(Character::DR_StepLeft); 
			mMoving = true;
		} 

		if (inputDevice->isKeyDown(KC_1))
		{
			if(mWeapon != 0)
			{
				mWeapon = 0;
				mForceAnimChange = true;
				if(mFireBall)
				 mFireBall->deactivate();

				OverlayElement* SwordIcon = OverlayManager::getSingleton().getOverlayElement("Sword");
				SwordIcon->setHeight(0.1);
				SwordIcon->setWidth(0.1);
				SwordIcon->setTop(0.89);
				SwordIcon->setLeft(0.1);

				OverlayElement* FBIcon = OverlayManager::getSingleton().getOverlayElement("FireBall");
				FBIcon->setHeight(0.04);
				FBIcon->setWidth(0.04);
				FBIcon->setTop(0.92);
				FBIcon->setLeft(0.22);
			}
		}
		if (inputDevice->isKeyDown(KC_2))
		{
			if(mWeapon!= 1)
			{
				mWeapon = 1;
				mForceAnimChange = true;
				if(mFireBall)
				 mFireBall->activate();

				OverlayElement* SwordIcon = OverlayManager::getSingleton().getOverlayElement("Sword");
				SwordIcon->setHeight(0.04);
				SwordIcon->setWidth(0.04);
				SwordIcon->setTop(0.92);
				SwordIcon->setLeft(0.13);

				OverlayElement* FBIcon = OverlayManager::getSingleton().getOverlayElement("FireBall");
				FBIcon->setHeight(0.1);
				FBIcon->setWidth(0.1);
				FBIcon->setTop(0.89);
				FBIcon->setLeft(0.18);
			}
		}

		if (inputDevice->isKeyDown(KC_LSHIFT))
		{
			mStrength -= 1.5 * mMaxStrength * 0.018;
				if(mStrength < 0.0) mStrength = 0.0;

			if(mStrength > 0.1 * mMaxStrength)
			{
				if(mCharacterController)
					((MyCharMoveController*)mCharacterController)->movespeed = 2.0;
				mRunning = true;				
			}
			else
			{
				if(mCharacterController)
					((MyCharMoveController*)mCharacterController)->movespeed = 1.0;
				mRunning = false;
			}
		}
		else
		{
			if(mCharacterController)
			((MyCharMoveController*)mCharacterController)->movespeed = 1.0;
			mRunning = false;
		}
	}
}

void FPSPlayer::processMouseInputs(InputReader* inputDevice)
{
	if(mHealth > 0)
	{
		if( inputDevice->getMouseButton( 0 ) )//attack
			setAnimState(PLAYERANIMSTATE_ATTACK);
			
		//else if( inputDevice->getMouseButton( 1 ) )//block
		//	setAnimState(PLAYERANIMSTATE_BLOCK);
	}
	else if( inputDevice->getMouseButton( 1 ))//respawn
	{
		mHealth = mMaxHealth;
		mStrength = mMaxStrength;
		mManna = mMaxManna;

		CompositorManager::getSingleton().setCompositorEnabled(OgreIlluminationManager::getSingleton().getMainViewport(),
														"GTP/PostProc/MotionBlur", false);
		this->mWeaponEntity->setVisible(true);
		this->mCharacterMeshEntity->setVisible(true);
		mWeapon = 0;
		isAlive = true;
	}

	Quaternion q; 
	q.FromAngleAxis( Radian(Degree(-inputDevice->getMouseRelX() * 0.13)), Vector3::UNIT_Y ); 
	q = mPlayerCharacter->getGlobalOrientation() * q; 
	mPlayerCharacter->setDirection(q); 

	mCamera->pitch(Radian(Degree(-inputDevice->getMouseRelY() * 0.13)));		
}

bool FPSPlayer::frameStarted(const FrameEvent& evt)
{
//	LogManager::getSingleton().logMessage("FPSPLAYER_FRAMESTARTED");

	float dt = evt.timeSinceLastFrame;
		
	if(mHealth == 0)//we are dead
	{
		CompositorManager::getSingleton().setCompositorEnabled(OgreIlluminationManager::getSingleton().getMainViewport(),
														"GTP/PostProc/MotionBlur", true);
		if(mFireBall)
			mFireBall->deactivate();
		static float dedanimT = 0;

		this->mWeaponEntity->setVisible(false);
		this->mCharacterMeshEntity->setVisible(false);

		float cameraPos = - dedanimT;
		if(cameraPos < -mCameraHeight) cameraPos = -mCameraHeight;
		
		mCamera->setPosition(Vector3(0,cameraPos,0));

		dedanimT += dt * 0.7;

		OverlayElement* healthUI = OverlayManager::getSingleton().getOverlayElement("HealthColumn");
		healthUI->setHeight(0);		
	}
	else
	{
		mStrength += mMaxStrength * 0.018;
		if(mStrength > mMaxStrength) mStrength = mMaxStrength;
		
		OverlayElement* strengthUI = OverlayManager::getSingleton().getOverlayElement("StrengthColumn");
		float relStrength = mStrength / mMaxStrength;
		strengthUI->setHeight(0.29 * relStrength);
		strengthUI->setTop(0.005 +  (1.0 - relStrength) * 0.29);

		mHealth += mMaxHealth * 0.005;
		if(mHealth > mMaxHealth) mHealth = mMaxHealth;
		
		OverlayElement* healthUI = OverlayManager::getSingleton().getOverlayElement("HealthColumn");
		float relHealth = mHealth / mMaxHealth;
		healthUI->setHeight(0.29 * relHealth);
		healthUI->setTop(0.005 +  (1.0 - relHealth) * 0.29);

		mManna += mMaxManna * 0.005;
		if(mManna > mMaxManna) mManna = mMaxManna;

		OverlayElement* mannaUI = OverlayManager::getSingleton().getOverlayElement("MannaColumn");
		float relManna = mManna / mMaxManna;
		mannaUI->setHeight(0.29 * relManna);
		mannaUI->setTop(0.005 +  (1.0 - relManna) * 0.29);

			
		if(mCharacterMesh_AnimState)
		{
			float animspeed = 1.2;
			if(mCurrentAnimState == PLAYERANIMSTATE_WALK && mRunning)
				animspeed *= 1.5;
			if(mCurrentAnimState == PLAYERANIMSTATE_ATTACK)
				animspeed *= 2.0;

			mCharacterMesh_AnimState->addTime(dt*animspeed);	
		}

		static float moveTime=0;
		
		float movecameraup = 0;
		if(mDebug)movecameraup = 2.0;

		if(mMoving)
		{
			setAnimState(PLAYERANIMSTATE_WALK);

			moveTime += dt;
			mCamera->setPosition(Vector3(0,mCameraHeight + movecameraup,0) + Vector3(0,0.009 * pow(sin(moveTime*5.0), 2),0));
		}
		else
		{
			if(Math::UnitRandom() > 0.98)
				setAnimState(PLAYERANIMSTATE_IDLE);
			else
				setAnimState(PLAYERANIMSTATE_STAND);

			moveTime = 0;
			mCamera->setPosition(Vector3(0,mCameraHeight + movecameraup,0));
		}

		if(mCharacterMeshEntity)
		this->mCharacterMeshEntity->_updateAnimation();

		if(mWeaponEntity)
		{
			mWeaponNode->setPosition(mWeaponAttachBone->getWorldPosition());
			mWeaponNode->setOrientation(mWeaponAttachBone->getWorldOrientation());

			Vector3 translation = mWeaponBoundNode->getWorldPosition();
			Quaternion orientation = mWeaponBoundNode->getWorldOrientation();
			
			mWeaponBound->setGlobalOrientation(orientation);		
			mWeaponBound->setGlobalPosition(translation);
			
			if(mDebug)
			{
				mWeaponBoundDebugNode->setPosition(translation);
				mWeaponBoundDebugNode->setOrientation(orientation);
			}
		}
		if(mFireBall)
			mFireBall->update(dt);	
	}
	return true;
}

String FPSPlayer::getAnimStateName(PlayerAnimState as, int option)
{
	switch (as)
	{
		case PLAYERANIMSTATE_IDLE:
		{
			if(mWeapon < 1)
				return "sword_idle";
			else
				return "magic_idle";
		}
		case PLAYERANIMSTATE_STAND:
		{
			if(mWeapon < 1)
				return "sword_stand";
			else
				return "magic_stand";
		}
		case PLAYERANIMSTATE_BLOCK:
		{
				return "block";			
		}
		case PLAYERANIMSTATE_WALK:
		{
				return "walk";			
		}
		case PLAYERANIMSTATE_ATTACK:
		{
			if(option)
			{
				if(mWeapon < 1)
					return "attack1";
				else
					return "magic_attack";
			}
			else
			{
				if(mWeapon < 1)
					return "attack2";
				else
					return "magic_attack";
			}
		}
	}
	return "";
}

void FPSPlayer::setAnimState(PlayerAnimState as)
{
	if(!mCharacterMeshEntity)
		return;

	switch(as)	
	{
		case PLAYERANIMSTATE_STAND:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_STAND || mForceAnimChange)
			{
				//wait until anim finishes
				if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop())
				{
					mCurrentAnimState = PLAYERANIMSTATE_STAND;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_STAND));
					mCharacterMesh_AnimState->setLoop(true);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_IDLE:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_IDLE || mForceAnimChange)
			{
				//wait until anim finishes
				if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop() || mCurrentAnimState == PLAYERANIMSTATE_IDLE)
				{
					mCurrentAnimState = PLAYERANIMSTATE_IDLE;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_IDLE));
					mCharacterMesh_AnimState->setTimePosition(0);
					mCharacterMesh_AnimState->setLoop(false);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_WALK:
		{
			if(mCurrentAnimState != PLAYERANIMSTATE_WALK)
			{
				if(mCurrentAnimState == PLAYERANIMSTATE_IDLE || mCurrentAnimState == PLAYERANIMSTATE_STAND)
				{
					mCurrentAnimState = PLAYERANIMSTATE_WALK;
					mCharacterMesh_AnimState->setEnabled(false);
					mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_WALK));
					mCharacterMesh_AnimState->setLoop(true);
					mCharacterMesh_AnimState->setEnabled(true);
				}
			}
			break;
		}
		case PLAYERANIMSTATE_ATTACK:
		{
			if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop() || mCurrentAnimState == PLAYERANIMSTATE_IDLE)
			{
			    int attackType = 0;
				if(Math::UnitRandom() > 0.5)
					 attackType = 1;

				mCurrentAnimState = PLAYERANIMSTATE_ATTACK;
				mCharacterMesh_AnimState->setEnabled(false);
				mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_ATTACK, attackType));
				mCharacterMesh_AnimState->setTimePosition(0);
				mCharacterMesh_AnimState->setLoop(false);
				mCharacterMesh_AnimState->setEnabled(true);

				if(mWeapon == 0)
				{
					for(std::set<PlayerCharacter*>::iterator it = mCharactersInRange.begin();
						it != mCharactersInRange.end(); it++)
						(*it)->addHit(getHit());
				}
				else
				{
					if(mManna > 20)
					{
						mManna -= 20;
						mStrength = 0;
						if(mFireBall)
							mFireBall->fire();
					}
				}
			}
			break;
		}		
		case PLAYERANIMSTATE_BLOCK:
		{
			if(mCharacterMesh_AnimState->getTimePosition() >=
					mCharacterMesh_AnimState->getLength() || mCharacterMesh_AnimState->getLoop() || mCurrentAnimState == PLAYERANIMSTATE_IDLE)
			{
				mCurrentAnimState = PLAYERANIMSTATE_BLOCK;
				mCharacterMesh_AnimState->setEnabled(false);
				mCharacterMesh_AnimState = mCharacterMeshEntity->getAnimationState(getAnimStateName(PLAYERANIMSTATE_BLOCK));
				mCharacterMesh_AnimState->setTimePosition(0);
				mCharacterMesh_AnimState->setLoop(false);
				mCharacterMesh_AnimState->setEnabled(true);
			}
			break;
		}
	}

	if(mForceAnimChange)
		mForceAnimChange = false;
}
